/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

#header {
    border-bottom: 0;
    margin-bottom: 6px;
    margin-top: 0;
    padding: 9px;
}
