# -*- coding: utf-8 -*-
# pylint: disable=unsubscriptable-object,unused-import
from anki.lang import _
# Form implementation generated from reading ui file 'designer/dyndconf.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(392, 472)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.limit = QtWidgets.QSpinBox(self.groupBox)
        self.limit.setMaximumSize(QtCore.QSize(60, 16777215))
        self.limit.setMinimum(1)
        self.limit.setMaximum(99999)
        self.limit.setObjectName("limit")
        self.gridLayout.addWidget(self.limit, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 2, 1, 1)
        self.search = QtWidgets.QLineEdit(self.groupBox)
        self.search.setObjectName("search")
        self.gridLayout.addWidget(self.search, 0, 1, 1, 4)
        self.order = QtWidgets.QComboBox(self.groupBox)
        self.order.setObjectName("order")
        self.gridLayout.addWidget(self.order, 1, 3, 1, 2)
        self.verticalLayout.addWidget(self.groupBox)
        self.filter2group = QtWidgets.QGroupBox(Dialog)
        self.filter2group.setObjectName("filter2group")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.filter2group)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_6 = QtWidgets.QLabel(self.filter2group)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 1, 0, 1, 1)
        self.order_2 = QtWidgets.QComboBox(self.filter2group)
        self.order_2.setObjectName("order_2")
        self.gridLayout_3.addWidget(self.order_2, 1, 3, 1, 2)
        self.search_2 = QtWidgets.QLineEdit(self.filter2group)
        self.search_2.setObjectName("search_2")
        self.gridLayout_3.addWidget(self.search_2, 0, 1, 1, 4)
        self.label_3 = QtWidgets.QLabel(self.filter2group)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 0, 0, 1, 1)
        self.limit_2 = QtWidgets.QSpinBox(self.filter2group)
        self.limit_2.setMaximumSize(QtCore.QSize(60, 16777215))
        self.limit_2.setMinimum(1)
        self.limit_2.setMaximum(99999)
        self.limit_2.setObjectName("limit_2")
        self.gridLayout_3.addWidget(self.limit_2, 1, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.filter2group)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 1, 2, 1, 1)
        self.verticalLayout.addWidget(self.filter2group)
        self.groupBox_2 = QtWidgets.QGroupBox(Dialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.resched = QtWidgets.QCheckBox(self.groupBox_2)
        self.resched.setChecked(True)
        self.resched.setObjectName("resched")
        self.gridLayout_2.addWidget(self.resched, 0, 0, 1, 2)
        self.secondFilter = QtWidgets.QCheckBox(self.groupBox_2)
        self.secondFilter.setObjectName("secondFilter")
        self.gridLayout_2.addWidget(self.secondFilter, 2, 0, 1, 1)
        self.previewDelayWidget = QtWidgets.QWidget(self.groupBox_2)
        self.previewDelayWidget.setObjectName("previewDelayWidget")
        self.previewDelayBox = QtWidgets.QHBoxLayout(self.previewDelayWidget)
        self.previewDelayBox.setObjectName("previewDelayBox")
        self.label_7 = QtWidgets.QLabel(self.previewDelayWidget)
        self.label_7.setObjectName("label_7")
        self.previewDelayBox.addWidget(self.label_7)
        self.previewDelay = QtWidgets.QSpinBox(self.previewDelayWidget)
        self.previewDelay.setObjectName("previewDelay")
        self.previewDelayBox.addWidget(self.previewDelay)
        self.label_8 = QtWidgets.QLabel(self.previewDelayWidget)
        self.label_8.setObjectName("label_8")
        self.previewDelayBox.addWidget(self.label_8)
        self.gridLayout_2.addWidget(self.previewDelayWidget, 1, 0, 1, 1)
        self.steps = QtWidgets.QLineEdit(self.groupBox_2)
        self.steps.setEnabled(False)
        self.steps.setObjectName("steps")
        self.gridLayout_2.addWidget(self.steps, 4, 0, 1, 1)
        self.stepsOn = QtWidgets.QCheckBox(self.groupBox_2)
        self.stepsOn.setObjectName("stepsOn")
        self.gridLayout_2.addWidget(self.stepsOn, 3, 0, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Help)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        self.secondFilter.toggled['bool'].connect(self.filter2group.setVisible)
        self.stepsOn.toggled['bool'].connect(self.steps.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.search, self.limit)
        Dialog.setTabOrder(self.limit, self.order)
        Dialog.setTabOrder(self.order, self.search_2)
        Dialog.setTabOrder(self.search_2, self.limit_2)
        Dialog.setTabOrder(self.limit_2, self.order_2)
        Dialog.setTabOrder(self.order_2, self.resched)
        Dialog.setTabOrder(self.resched, self.previewDelay)
        Dialog.setTabOrder(self.previewDelay, self.secondFilter)
        Dialog.setTabOrder(self.secondFilter, self.stepsOn)
        Dialog.setTabOrder(self.stepsOn, self.steps)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_("Dialog"))
        self.groupBox.setTitle(_("Filter"))
        self.label_5.setText(_("Limit to"))
        self.label_2.setText(_("Search"))
        self.label.setText(_("cards selected by"))
        self.filter2group.setTitle(_("Filter 2"))
        self.label_6.setText(_("Limit to"))
        self.label_3.setText(_("Search"))
        self.label_4.setText(_("cards selected by"))
        self.groupBox_2.setTitle(_("Options"))
        self.resched.setText(_("Reschedule cards based on my answers in this deck"))
        self.secondFilter.setText(_("Enable second filter"))
        self.label_7.setText(_("Repeat failed cards after"))
        self.label_8.setText(_("minutes"))
        self.steps.setText(_("1 10"))
        self.stepsOn.setText(_("Custom steps (in minutes)"))

