/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

.field {
    border: 1px solid #aaa;
    background: #fff;
    color: #000;
    padding: 5px;
    overflow-wrap: break-word;
}

.clearfix:after {
    content: "";
    display: table;
    clear: both;
}

.fname {
    vertical-align: middle;
    padding: 0;
}

img {
    max-width: 90vw;
}

body {
    margin: 5px;
}

#topbutsOuter {
    position: fixed;
    top: 0;
    padding: 2px;
    left: 0;
    right: 0;
}

.topbut {
    width: 16px;
    height: 16px;
    margin-top: 4px;
}

.rainbow {
    background-image: -webkit-gradient(linear, left top, left bottom,
    color-stop(0.00, #f77),
    color-stop(50%, #7f7),
    color-stop(100%, #77f));
}

.linkb {
    -webkit-appearance: none;
    border: 0;
    padding: 0px 2px;
    background: transparent;
}

.linkb:disabled {
    opacity: 0.3;
    cursor: not-allowed;
}

.highlighted {
    border-bottom: 3px solid #000;
}

#fields {
    margin-top: 35px;
}
