/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

body {
    margin: 0px;
    padding: 0;
}

button {
    min-width: 60px;
    white-space: nowrap;
    margin: 0.5em;
}

.hitem {
    margin-top: 2px;
}

.stat {
    padding-top: 5px;
}

.stat2 {
    padding-top: 3px;
    font-weight: normal;
}

.stattxt {
    padding-left: 5px;
    padding-right: 5px;
    white-space: nowrap;
}

#ansbut { margin-bottom: 1em; }

.nobold {
    font-weight: normal;
    display: inline-block;
    padding-top: 4px;
}

.spacer {
    height: 18px;
}

.spacer2 {
    height: 16px;
}

#outer {
    border-top: 1px solid #aaa;
}

#innertable {
    padding: 3px;
}
