/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

a.deck {
    color: #000;
    text-decoration: none;
    min-width: 5em;
    display: inline-block;
}

a.deck:hover {
    text-decoration: underline;
}

tr.deck td {
    border-bottom: 1px solid #e7e7e7;
}

tr.top-level-drag-row td {
    border-bottom: 1px solid transparent;
}

td {
    white-space: nowrap;
}

tr.drag-hover td {
    border-bottom: 1px solid #aaa;
}

body {
    margin: 1em;
    -webkit-user-select: none;
}

.current {
    background-color: #e7e7e7;
}

.decktd {
    min-width: 15em;
}

.count {
    min-width: 4em;
    text-align: right;
}

.optscol {
    width: 2em;
}

.collapse {
    color: #000;
    text-decoration: none;
    display: inline-block;
    width: 1em;
}

.filtered {
    color: #00a !important;
}

.gears {
    width: 1em;
    height: 1em;
    opacity: .5;
    padding-top: 0.2em;
}
