# This file auto-generated by build_ui.sh. Don't edit.
__all__ = [
	"about",
	"addcards",
	"addfield",
	"addmodel",
	"addonconf",
	"addons",
	"browser",
	"browserdisp",
	"browseropts",
	"changemap",
	"changemodel",
	"clayout_top",
	"customstudy",
	"dconf",
	"debug",
	"dyndconf",
	"editaddon",
	"editcurrent",
	"edithtml",
	"exporting",
	"fields",
	"finddupes",
	"findreplace",
	"getaddons",
	"importing",
	"main",
	"modelopts",
	"models",
	"preferences",
	"preview",
	"profiles",
	"progress",
	"reposition",
	"reschedule",
	"setgroup",
	"setlang",
	"stats",
	"studydeck",
	"taglimit",
	"template",
]
from . import about
from . import addcards
from . import addfield
from . import addmodel
from . import addonconf
from . import addons
from . import browser
from . import browserdisp
from . import browseropts
from . import changemap
from . import changemodel
from . import clayout_top
from . import customstudy
from . import dconf
from . import debug
from . import dyndconf
from . import editaddon
from . import editcurrent
from . import edithtml
from . import exporting
from . import fields
from . import finddupes
from . import findreplace
from . import getaddons
from . import importing
from . import main
from . import modelopts
from . import models
from . import preferences
from . import preview
from . import profiles
from . import progress
from . import reposition
from . import reschedule
from . import setgroup
from . import setlang
from . import stats
from . import studydeck
from . import taglimit
from . import template
