/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

hr {
    background-color: #ccc;
}

body {
    margin: 20px;
    overflow-wrap: break-word;
}

body.nightMode {
    background-color: black;
    color: white;
}

img {
    max-width: 95vw;
    max-height: 95vh;
}

#_flag {
    position: fixed;
    right: 10px;
    top: 0;
    font-size: 30px;
    display: none;
    -webkit-text-stroke-width: 1px;
    -webkit-text-stroke-color: black;
}

#_mark {
    position: fixed;
    left: 10px;
    top: 0;
    font-size: 30px;
    color: yellow;
    display: none;
    -webkit-text-stroke-width: 1px;
    -webkit-text-stroke-color: black;
}

#typeans {
    width: 100%;
}

.typeGood {
    background: #0f0;
}

.typeBad {
    background: #f00;
}

.typeMissed {
    background: #ccc;
}
