/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

.smallLink {
    font-size: 10px;
}

h3 {
    margin-bottom: 0;
}

.descfont {
    padding: 1em;
    color: #333;
}

.description {
    white-space: pre-wrap;
}

#fulldesc {
    display: none;
}

.descmid {
    width: 70%;
    margin: 0 auto 0;
    text-align: left;
}

.dyn {
    text-align: center;
}
